/****************************************************************************
*  FILENAME: swap.c   PRODUCT NAME: CRYPTOGRAPHIC TOOLKIT
*
*  FILE STATUS:
*
*  DESCRIPTION: Byte and Word Swap functions 
*
*  PUBLIC FUNCTIONS:
*
*
*  PRIVATE FUNCTIONS:
*
*  REVISION  HISTORY:
*
*  14 Oct 94   GKL     Initial release
*  26 Oct 94   GKL     (alignment for big endian support )
*
****************************************************************************/

/****************************************************************************
*  INCLUDE FILES
****************************************************************************/
/* bn files */
#include "bn.h"
#include "bn32.h"
/* system files */
#ifdef VXD
#include <vtoolsc.h>
#else
#include <stdlib.h>
#include <string.h>
#endif
/* program files */
#include "cylink.h"
#include "endian.h"
#include "desext.h"

/*Reset bytes in long*/
/*extern void ByteSwap32_asm( uchar  *X, ushort X_len );*/ /*kz*/

/****************************************************************************
*  NAME:     void ByteSwap32 (uchar  *array,
*                             ushort X_len )
*
*  DESCRIPTION:  Perform byte reversal on an array of longword.
*
*  INPUTS:
*          PARAMETERS:
*            uchar  *X            Pointer to array
*                        ushort X_len             Number of bytes
*  OUTPUT:
*            uchar  *X            Pointer to array
*
*  REVISION HISTORY:
*
*  24 Sep 94    KPZ             Initial release
*  14 Oct 94    GKL     Second version (big endian support)
*
****************************************************************************/


void ByteSwap32_big( uchar  *X, ushort X_len )
{
/*#if defined ( ASM_FUNCTION )  && defined( L_ENDIAN )
  ByteSwap32_asm ( X, X_len);
#else*/ /*kz*/
   ushort i;        /*counter*/
    uchar a;      /*temporary char*/
        for ( i = 0; i < X_len; i += 4)
 {
               a = X[i];
               X[i] = X[i+3];
          X[i+3] = a;
             a = X[i+1];
             X[i+1] = X[i+2];
                X[i+2] = a;
     }
/*#endif*/ /*kz*/
}


/****************************************************************************
*  NAME:     void ByteSwap (uchar  *array,
*                           ushort X_len )
*
*  DESCRIPTION:  Perform byte reversal on an array of longword or shortword.
*
*  INPUTS:
*          PARAMETERS:
*            uchar  *X            Pointer to array
*            ushort X_len             Number of bytes
*  OUTPUT:
*            uchar  *X            Pointer to array
*
*  REVISION HISTORY:
*
*  24 Sep 94    KPZ             Initial release
*  14 Oct 94    GKL     Second version (big endian support)
*
****************************************************************************/

void ByteSwap( uchar  *X,
               ushort X_len )
{
#ifdef ORD_16
    ushort i;        /*counter*/
    uchar a;      /*tempriory char for revers*/
	for ( i = 0; i < X_len; i += 2)
    {
		a = X[i];
        X[i] = X[i+1];
        X[i+1] = a;
    }
#endif
#ifdef ORD_32
      ByteSwap32_big(X,X_len);
#endif
}

/*kz longbyte deleted */

/****************************************************************************
*  NAME:     void WordSwap (uchar  *array,
*                           ushort X_len )
*
*  DESCRIPTION:  Perform short reversal on an array of longword.
*
*  INPUTS:
*          PARAMETERS:
*            uchar  *X            Pointer to array
*                        ushort X_len             Number of bytes
*  OUTPUT:
*            uchar  *X            Pointer to array
*
*  REVISION HISTORY:
*
*  14 Oct 94    GKL     Initial release
*
****************************************************************************/
void WordSwap( uchar  *X,
               ushort X_len )
{
	ushort i;        /*counter*/
    ushort a;      /*tempriory ushort*/

    for ( i = 0; i < X_len; i += 4)
    {
        a = *(ushort*)(&X[i]);
        *(ushort*)(&X[i])=*(ushort*)(&X[i+2]);
        *(ushort*)(&X[i+2])=a;
    }
}

